import { enrichReducedReportWithBaseline } from '../../collect/utils/report/utils.js';
import { table } from '../../../core/md/table.js';
import { style } from '../../../core/md/font-style.js';
import { headline } from '../../../core/md/headline.js';
import { NEW_LINE } from '../../../core/md/constants.js';
export async function generateMdReport(flowResult) {
    const name = flowResult.name;
    const dateTime = `Date/Time: ${style(new Date().toISOString().replace('T', ' ').split('.')[0].slice(0, -3))}  `;
    const stepsTable = await getStepsTable(flowResult);
    return `${headline(name)}${NEW_LINE}
${dateTime}${NEW_LINE}
${stepsTable}${NEW_LINE}
`;
}
/**
 * | Step Name       | Gather Mode |Performance | Accessibility | BestPractices | Seo  | PWA |
 * | --------------- | ----------- | ------------- | ------------- | ---- | --- |
 * |  Nav1           |  99         | 50            | 100           | 98   |  -  |
 * |  Snap   1       |  3/3        | 22/5          | 5/2           | 7/10 |  -  |
 * |  TimeSpan 1     |  10/11      | -             | 4/7           | 7/10 |  -  |
 */
export async function getStepsTable(reducedReport, baselineResults) {
    const reportCategories = Object.keys(reducedReport.steps[0].results);
    const tableStepsArr = formatStepsForMdTable(reportCategories, reducedReport, baselineResults);
    const alignOptions = headerAlignment(reportCategories);
    const tableArr = extractTableArr(reportCategories, tableStepsArr);
    return await table(tableArr, alignOptions);
}
function formatStepsForMdTable(reportCategories, reducedReport, baselineResults) {
    if (baselineResults) {
        const enrichedReducedReport = enrichReducedReportWithBaseline(reducedReport, baselineResults);
        return enrichedReducedReport.steps.map((step) => {
            const results = reportCategories.map(category => extractEnrichedResults(step, category));
            return [step.name, step.gatherMode].concat(results);
        });
    }
    return reducedReport.steps.map((step) => {
        const results = reportCategories.map(category => extractResultsValue(step.results[category]));
        return [step.name, step.gatherMode].concat(results);
    });
}
function extractTableArr(reportCategories, steps) {
    const tableHead = extractTableHead(reportCategories);
    return [tableHead].concat(steps);
}
function extractTableHead(reportCategories) {
    return ['Step Name', 'Gather Mode']
        .concat(reportCategories.map(c => c.split('-').map(cN => cN[0].toUpperCase() + cN.slice(1)).join(' ')));
}
function extractFractionalResultValue(fractionResults) {
    const { totalWeight, numPassed, numPassableAudits } = fractionResults;
    const value = numPassed + '/' + numPassableAudits;
    return totalWeight === 0 ? 'Ø ' + value : value;
}
function extractResultsValue(stepResult) {
    if (typeof stepResult === 'number') {
        return (stepResult * 100).toFixed(0);
    }
    return stepResult ? extractFractionalResultValue(stepResult) : '-';
}
function extractEnrichedResults(step, category) {
    const result = extractResultsValue(step.results[category]);
    const baseline = extractResultsValue(step.baseline[category]);
    return resultWithBaselineComparison(result, baseline);
}
function resultWithBaselineComparison(result, baseline) {
    if (result === baseline) {
        return result;
    }
    const resultNum = Number(result.replace('Ø ', '').split('/')[0]);
    const baselineNum = Number(baseline.replace('Ø ', '').split('/')[0]);
    const difference = baselineNum - resultNum;
    return `${result} (${difference > 0 ? '+' : ''}${difference})`;
}
function headerAlignment(reportCategories) {
    const reportFormats = reportCategories.map(_ => 'c');
    return ['l', 'c'].concat(reportFormats);
}
//# sourceMappingURL=md-report.js.map