import { existsSync, lstatSync, mkdirSync, readFileSync, writeFileSync } from 'fs';
import { pathToFileURL } from 'node:url';
import { register } from 'node:module';
import { dirname, join } from 'node:path';
import { logVerbose } from '../loggin/index.js';
import { formatCode, getParserFromExtname } from '../prettier/index.js';
import { cwd } from 'node:process';
export { toFileName } from './to-file-name.js';
function jsonParse(str) {
    return JSON.parse(str);
}
/*
const w = readFile('path') // string
const x = readFile('path', {}) // string
const y = readFile('path', {ext: 'json'}) // {}
const z = readFile<{ n: number }>('path', {ext: 'json'}) // {n: number}
*/
/**
 * Ensures the file exists before reading it
 */
export function readFile(path, cfg) {
    const { fail, ext } = { fail: false, ...cfg };
    if (!existsSync(path)) {
        const errorStr = `${path} does not exist.`;
        if (!fail) {
            logVerbose(errorStr);
            return '';
        }
        throw new Error(errorStr);
    }
    if (lstatSync(path).isDirectory()) {
        throw new Error(`${path} is a directory but needs to be a file.`);
    }
    const fileContent = readFileSync(path, 'utf-8');
    return ext === 'json' ? jsonParse(fileContent) : fileContent;
}
/**
 * Ensures the folder exists before writing it
 */
export async function writeFile(filePath, data) {
    const dir = dirname(filePath);
    if (!existsSync(dir)) {
        logVerbose(`Created dir ${dir} to save ${filePath}`);
        mkdirSync(dir);
    }
    const ext = filePath.split('.').pop() || '';
    const formattedData = await formatCode(data, getParserFromExtname(ext));
    // @TODO implement a check that saves the file only if the content is different => git noise
    return writeFileSync(filePath, formattedData);
}
export async function resolveAnyFile(path) {
    if (path.endsWith('.mts')) {
        register('tsx/esm', {
            parentURL: import.meta.url,
            data: true
        });
    }
    const file = await import(pathToFileURL(join(cwd(), path)).pathname);
    // If the user provides a configuration in TS file
    // then there are 2 cases for exporting an object. The first one is:
    // `module.exports = { ... }`. And the second one is:
    // `export default { ... }`. The ESM format is compiled into:
    // `{ default: { ... } }`
    const exports = file.default || file;
    return { exports, path };
}
//# sourceMappingURL=index.js.map